package jeu.echec.pieces.echiquier;

import java.util.*;

import jeu.echec.pieces.*;

/**
 * Représente un échiquier avec toutes ses pièces
 * @author mouss
 *
 */
public class Echequier {
	private Boolean Jeu;
    private Piece[][] echequier = new Piece[8][8];
    private Position pos = new Position();
    Scanner user_input = new Scanner(System.in);
    private static int departLigne, departColonne, arriveeligne, arriveColonne;
    private static Boolean Tourdujoueur;
    // Définit sur true si movement est invalide. Demande de nouveau l'utilisateur dans move ()
    // méthode.
    private static Boolean MouvementIncorrect = false;
    // Contient une chaîne avec l'entrée de l'utilisateur pour les instructions de déplacement
    String ledepart, larrivee, mouvement, ledeparte;
 
 
    /**
	* Construit un échiquier avec ses pièces à leur position initiale.
	*/
    public Echequier() {
 
        initialisation_de_la_grille(echequier);
        Jeu = true;
 
    }
 
    /**
     * Si jeu est true on returne le jeu
     * @return
     */
    public Boolean getJeu() {
        return this.Jeu;
    }
    
 
    /** 
     * Initialisation de la grille avec toutes les pièces
     * 
     */
    private static void initialisation_de_la_grille(Piece[][] echequier) {
        // un échiquier avec une matrice 8x8 de pièces
        // les lignes [0] et [1] sont noires
        // les lignes [6] et [7] sont blanches
 
        for (int ligne = 0; ligne < echequier.length; ligne++) {
            for (int colonne = 0; colonne < echequier[ligne].length; colonne++) {
            	// Toutes les pièces noires sauf les pions noires sur l'échiquier  
                if (ligne == 0) {
                    switch (colonne) {
                    case 0:echequier[ligne][colonne] = new Tour(false);break;
                    case 1:echequier[ligne][colonne] = new Cavalier(false);break;
                    case 2:echequier[ligne][colonne] = new Fou(false);break;
                    case 3:echequier[ligne][colonne] = new Reine(false);break;
                    case 4:echequier[ligne][colonne] = new Roi(false);break;
                    case 5:echequier[ligne][colonne] = new Fou(false);break;
                    case 6:echequier[ligne][colonne] = new Cavalier(false);break;
                    case 7:echequier[ligne][colonne] = new Tour(false);break;
                    }
                } 
                // Toutes les pions noires sur l'échiquier
                else if (ligne == 1) {
                    echequier[ligne][colonne] = new Pion(false);
                } 
                // Toutes les pions blanches sur l'échiquier
                else if (ligne == 6) {
                    echequier[ligne][colonne] = new Pion(true);
                } 
                // Toutes les pièces blanches sauf les pions blanches sur l'échiquier
                else if (ligne == 7) {
                    switch (colonne) {
                    case 0:echequier[ligne][colonne] = new Tour(true);break;
                    case 1:echequier[ligne][colonne] = new Cavalier(true);break;
                    case 2:echequier[ligne][colonne] = new Fou(true);break;
                    case 3:echequier[ligne][colonne] = new Reine(true);break;
                    case 4:echequier[ligne][colonne] = new Roi(true);break;
                    case 5:echequier[ligne][colonne] = new Fou(true);break;
                    case 6:echequier[ligne][colonne] = new Cavalier(true);break;
                    case 7:echequier[ligne][colonne] = new Tour(true);break;
                    }
                } else {
                    echequier[ligne][colonne] = null;
                }
            }
        }
 
        // Affecte aléatoirement qui commence en premier (noir ou blanc)
        Random rand = new Random();
        Tourdujoueur = rand.nextBoolean();
 
    }
 
    /**
	* Affichage de la grille et les pièces.
	*/
    public void grille() {
         

        System.out.println(" ┬──┬──┬──┬──┬──┬──┬──┬──┐");
        for (int ligne = 0; ligne < echequier.length; ligne++) {
            System.out.print(8 - ligne + "│");
            for (int colonne = 0; colonne < echequier[ligne].length; colonne++) {
                if (echequier[ligne][colonne] != null) {
                    echequier[ligne][colonne].dessiner();
                    System.out.print("│");
 
                }
                else {
                    System.out.print("  │");
                         
                }
            }
            System.out.print(" \n");
            System.out.print(" ┼──┼──┼──┼──┼──┼──┼──┼──┤ \n");
        }
        System.out.println("   a  b  c  d  e  f  g  h");
    }
 
    /**
	* Retourne toutes les mouvements à valider après avoir jouer.
	*/
    private boolean mouvementValide(boolean Echec) {
 
        // On verifie que le chemin ne sort pas de l'échequier
        if ((departLigne < 0 && departLigne > 7) || (departColonne < 0 && departColonne > 7) || (arriveeligne < 0 && arriveeligne > 7) || (arriveeligne < 0 && arriveColonne > 7)) {
            System.out.println("Le mouvement est en dehors de l'echequier");
            return false;
        }
        	

 
        // On vérifie que l'origine n'est pas null
        if (echequier[departLigne][departColonne] == null) {
            System.err.println("L'origine est vide");
            return false;
        }
 
        // On verifie que c'est bien le tour du joueur pour qu'il joue
        if ((echequier[departLigne][departColonne].estBlanc() && !Tourdujoueur) || (!echequier[departLigne][departColonne].estBlanc() && Tourdujoueur)) {
            if(Echec == false) 
            System.err.println("Ce n'est pas votre tour !");
            return false;
        }
 
        // On verifie que les déplacement spécifique de la pièce sont bien respectés
        if (!echequier[departLigne][departColonne].deplacementValide(departLigne, departColonne, arriveeligne,arriveColonne)) {
            if(Echec == false) 
                System.err.println("Cette pièce ne bouge pas comme ça");
            return false;
        }
 
        // cette déclaration arrête la déclaration pour vérifier si le blanc atterit
        // blanc de l'exécution estBlanc() sur un espace null
        if (echequier[arriveeligne][arriveColonne] == null) {          
            //Chemin pas possible si une piece veut fauter une autre pièce SAUF POUR LE CAVALIER
            if(!(echequier[departLigne][departColonne] instanceof Cavalier)){
                int deplaceX = 0, deplaceY = 0;
                if(departLigne < arriveeligne) deplaceX = 1; else if(departLigne > arriveeligne)deplaceX = -1;
                if(departColonne < arriveColonne) deplaceY = 1; else if(departColonne > arriveColonne) deplaceY = -1;
 
                int i = departLigne, j = departColonne;
                
                while(i != arriveeligne || j != arriveColonne){
                    if(i != arriveeligne) i+= deplaceX;
                    if(j != arriveColonne) j+= deplaceY;
 
                    if(echequier[i][j] != null){
                        System.err.println("Cette pieces ne peut pas sauter un pion");
                        return false;
                    }
                }
            }
            return true;
        }
        
        // On verifie que un pion ne veut pas de déplacer en diagonale
        if(echequier[departLigne][departColonne] instanceof Pion && (arriveeligne == arriveColonne) && this.echequier[arriveeligne][arriveColonne] == null ){
            System.out.println("Ne peut pas ce deplacer en diagonale");
            return false;
        }
 
        // On verifie que un pion Blanc ne veut pas se poser sur un autre pion Blanc
        if (echequier[departLigne][departColonne].estBlanc() && echequier[arriveeligne][arriveColonne].estBlanc()) {
            System.err.println("Les blancs ne peuvent pas atterrir sur le blanc");
            return false;
        }
 
        // On verifie que un pion Noir ne veut pas se poser sur un autre pion Noir
        if (!echequier[departLigne][departColonne].estBlanc() && !echequier[arriveeligne][arriveColonne].estBlanc()) {
            System.err.println("Le noir ne peut pas atterrir sur le noir");
            return false;
        }
         
        return true;
    }
    
    /**
	* Affiche les pièces jouables d'un joueur.
	*/
    public void piecesjouables(){
    	System.out.println("Pièces jouables :");
    	
    	// Liste de toutes les pièces
    	ArrayList<Piece> piecesDeplacables = new ArrayList<Piece>();
    	
    	// On verifie que un pion ne veut pas de déplacer en diagonale
    	for(int i = 0; i<this.echequier.length; i++) {
    		int CoordLigne = 8-i;
        	for(int j = 0; j<this.echequier.length; j++) {
        		int CoordColonne = j;
        		
        		boolean estDeplacable = false;
        		
        		// Verifie si une case est dispo
				int x = i, y = j;

                    // Vérifie si une case n'est pas vide
    				if(this.echequier[i][j] != null) {
    					//Vérifie les déplacements possibles 
               			if(!this.echequier[i][j].deplacementValide(i, j, x, y)) {
           					// Vérifie le joueur jouant
           					// Joueur Blanc
           					if (this.Tourdujoueur && this.echequier[i][j].estBlanc()) {
           						estDeplacable = true;                    			            
           			        } 
           					// Joueur Noir
           					if (!this.Tourdujoueur && !this.echequier[i][j].estBlanc()) {
           			        	estDeplacable = true;
           			        }
               			}
           			}
        		//Ajoute la piece a une liste si elle a des cases vide
        		if(estDeplacable) {
        			piecesDeplacables.add(this.echequier[i][j]);
        			this.echequier[i][j].setPosition(CoordLigne, CoordColonne);
        		}
        	}
        }
    	
    	//Parcours les pièces jouables
    	for(Piece p : piecesDeplacables) {
    		System.out.print(p.toString()+ " --> " ); p.dessiner();
    		System.out.println("");   		
    	}
    }
    
    
    /**
     * Affiche les déplacements possibles d'une pièce jouable.
     * @param x
     * @param y
     * @param coord
     */
    public void deplacementspossibles(int x, int y, String coord) {
    	System.out.println("Vers quelle case souhaitez-vous déplacer le "+this.echequier[x][y].getClass().getSimpleName()+" situé en "+coord+" :");    	
    	for(int i = 0; i<this.echequier.length; i++) {
    		int CoordLigne = 8-i;
    		for(int j = 0; j<this.echequier.length; j++) {
    			int CoordColonne = j;
    			int m = i, n = j;
    			// Vérifie si une case est vide
				if(this.echequier[i][j] == null) {
					// Vérifie si une pièce peut se déplacer sans atterir sur une pièce du même camp.
           			if(this.echequier[x][y].deplacementValide(x, y, i, j)) {
           				//Verifion si c'est un pion 
           				if(this.echequier[x][y] instanceof Pion){
           					//Déplacement verticale
           					if((x<i+1 || x<i+2 || x>2-i || x>1-i) && y==j && echequier[m][n] == null) {
           						pos.positionDeplacer(CoordLigne, CoordColonne);
           					}
           					//Déplacement diagonale si possible
           					if(echequier[m][n] != null){
           						pos.positionDeplacer(CoordLigne, CoordColonne);
           					}
           				} 
           				if(!(this.echequier[x][y] instanceof Pion)){
           		        	pos.positionDeplacer(CoordLigne, CoordColonne);
           		        }
           			}
       			}
    		} 		
    	}
    }
 
    /**
	* Effectue le déplacement d'une pièce lorsque un joueur est en train de jouer.
	*/
    public void deplacer() {
        if (MouvementIncorrect) {
            System.err.println("Le mouvement n'est pas valide. Veuillez réessayer:");
            MouvementIncorrect = false;
        }
 
        else if (Tourdujoueur) {
            System.out.println( "\nC'est à BLANC de jouer !");
        } else {
            System.out.println("\nC'est à NOIR de jouer !");
        }
        
        //Afichage des pièces jouables 
        piecesjouables();
        System.out.println("Choisissez une pièce :");
        ledepart = user_input.nextLine();
        ledeparte = ledepart+" a ";
        
     // convertir en minuscules
        String miniScule = ledeparte.toLowerCase();
        
        // analyse la chaîne de déplacement en composants
        String[] composant = miniScule.split(" ");
 
        // On utilise des chars dans les composants pour définir les coordonnées du tableau pour le déplacement origine et le déplacement destination
        departLigne = 7 - (composant[0].charAt(1) - '1');
        departColonne = composant[0].charAt(0) - 'a';
        
        //Affichage des cases disponibles
        deplacementspossibles(departLigne, departColonne, ledepart);
        
        System.out.println("Choisissez une case :");
        larrivee = user_input.nextLine();
        mouvement = ledepart+" a "+larrivee;
        
        // convertir en minuscules
        String lowerCase = mouvement.toLowerCase();
        
        // analyse la chaîne de déplacement en composants
        String[] components = lowerCase.split(" ");
 
        // On utilise des chars dans les composants pour définir les coordonnées du tableau pour le déplacement origine et le déplacement destination
        departLigne = 7 - (components[0].charAt(1) - '1');
        departColonne = components[0].charAt(0) - 'a';
        arriveeligne = 7 - (components[2].charAt(1) - '1');
        arriveColonne = components[2].charAt(0) - 'a';
        
        if (mouvementValide(false)) {
        	// Message après avoir déplacer une pièce
        	System.out.println("\nVous avez choisi de déplacer le "+this.echequier[departLigne][departColonne].getClass().getSimpleName()+" "+ledepart+" vers "+larrivee+".");
            // mettre une pièce dans la destination
            echequier[arriveeligne][arriveColonne] = echequier[departLigne][departColonne];
            // vide l'origine du mouvement
            echequier[departLigne][departColonne] = null;
            // Tour de l'autre joueur
            Tourdujoueur = !Tourdujoueur;
        }
        else {
            MouvementIncorrect = true;
            mouvementValide(false);
        }
 
    }
    
    
 
}